#ifndef _DICTIONARYSYSTEMH_
#define _DICTIONARYSYSTEMH_

#ifdef INFORMATION
Copyright (C) 2011 by Bruce Wilcox

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#endif


typedef unsigned int DICTINDEX;	//   indexed ref to a dictionary entry

//   This file has formatting restrictions because it is machine read by AcquireDefines. All macro and function calls
//   must keep their ( attached to the name and all number defines must not.
//   All tokens of a define must be separated by spaces ( TEST|TEST1  is not legal).

//   These are the properties of a dictionary entry.
//   IF YOU CHANGE THE VALUES or FIELDS of a dictionary node,
//   You need to rework ReadDictionary/WriteDictionary/ReadBinaryEntry/WriteBinaryEntry

// D->properties bits

//   BASIC_POS POS bits like these must be on the bottom of any 64-bit word, so they match MEANING values (at top) as well.
#define NOUN					0x0000000080000000ULL
#define VERB					0x0000000040000000ULL
#define ADJECTIVE				0x0000000020000000ULL	// PENNBANK: JJ when not JJR or JJS
#define ADVERB					0x0000000010000000ULL   // PENNBANK: RB when not RBR or RBS
#define BASIC_POS				( NOUN | VERB | ADJECTIVE | ADVERB )
//   the above four should be first, reflecting the Wordnet files (leaving room for wordnet offset)
#define PREPOSITION				0x0000000008000000ULL  // Pennbank: IN
#define ESSENTIAL_FLAGS			( BASIC_POS | PREPOSITION ) //  these can be type restrictions on MEANINGs
#define DETERMINER				0x0000000004000000ULL //   Articles: a and the --  demonstratives: this that these those --  Quantifiers: much many some a_little  -- Possessive pronouns: my your
// FLAGS above are shared onto systemflags for most likely pos-tag candidate type

#define FOREIGN_WORD			0x0000000002000000ULL 
#define CONJUNCTION_COORDINATE	0x0000000001000000ULL // Pennbank: CC (coordinating)  IN (subordinating)  may also be interjection
#define CONJUNCTION_SUBORDINATE	0x0000000000800000ULL 

#define COMMA					0x0000000000400000ULL 

#define POSSESSIVE				0x0000000000200000ULL // is a possessive like 's or Taiwanese (but not possessive pronoun)

#define ADJECTIVE_BASIC			0x0000000000100000ULL	// "friendly"
#define ADJECTIVE_MORE			0x0000000000080000ULL	// "friendlier"
#define ADJECTIVE_MOST			0X0000000000040000ULL	// "friendliest"
#define ADJECTIVE_PARTICIPLE	0x0000000000020000ULL	// the "walking" dead  or  the "walked" dog
#define ADJECTIVE_CARDINAL		0x0000000000010000ULL	// "20" horses
#define ADJECTIVE_ORDINAL		0x0000000000008000ULL	// "to" fly

#define ADVERB_BASIC			0x0000000000004000ULL	
#define ADVERB_MORE				0x0000000000002000ULL	 
#define ADVERB_MOST				0x0000000000001000ULL	

#define THERE_EXISTENTIAL		0x0000000000000800ULL  // "There" is no future in it.

#define PRONOUN_SUBJECT			0x0000000000000400ULL	// I you we
#define PRONOUN_POSSESSIVE		0x0000000000000200ULL	// my your his
#define PRONOUN_OBJECT			0x0000000000000100ULL	// me, you, him, her, etc

#define NOUN_SINGULAR			0x0000000000000080ULL  // Pennbank: NN
#define NOUN_PLURAL				0x0000000000000040ULL  // Pennbank: NNS
#define NOUN_PROPER_SINGULAR	0x0000000000000020ULL  //   A proper noun that is NOT a noun is a TITLE like Mr.
#define PAREN					0x0000000000000010ULL  // ( or )
#define NOUN_GERUND				0x0000000000000008ULL	// "Walking" is fun
#define NOUN_CARDINAL			0x0000000000000004ULL  // I followed the "20".
#define NOUN_PROPER_PLURAL		0x0000000000000002ULL	

#define WH_ADVERB				0x0000000000000001ULL	// adverbs- some can introduce an interrogative sentence "who is there" and others can introduce clauses "I've no idea how it works"


//////////////////////////////16 bits below here can not be used in values[] of tagger  
#define AS_IS					0x8000000000000000ULL   //  TRANSIENT INTERNAL dont try to reformat the word (transient flag passed to StoreWord)
#define NOUN_HUMAN				0x4000000000000000ULL  //   person or group of people that uses WHO, he, she, anyone
#define NOUN_FIRSTNAME			0x2000000000000000ULL  //   a known first name -- wiULL also be a sexed name probably
#define NOUN_SHE				0x1000000000000000ULL	//   female sexed word (used in sexed person title detection for example)
#define NOUN_HE					0x0800000000000000ULL	//   male sexed word (used in sexed person title detection for example)
#define NOUN_THEY				0x0400000000000000ULL   
#define NOUN_TITLE_OF_ADDRESS	0x0200000000000000LL	//   eg. mr, miss
#define NOUN_TITLE_OF_WORK		0x0100000000000000ULL
#define LOWERCASE_TITLE			0X0080000000000000ULL	//   lower case word may be in a title (but not a noun)
#define NOUN_ABSTRACT			0x0040000000000000ULL	// can be an abstract noun (maybe also concrete)
#define WORDNET_ID				0x0020000000000000ULL	//   a wordnet synset header node (MASTER w gloss )
#define QWORD					0x0010000000000000ULL	// who what where why when how whose
#define NOUN_ROLE				0x0008000000000000ULL	//  human role (like doctor or pilot or ally)
#define UPPERCASE_WORDHASH		0x0004000000000000ULL	// treat as uppercase
#define NOUN_MASS				0x0002000000000000ULL	// tagger must use POSSIBLEMASSNOUN test since bit not available
#define PUNCTUATION				0x0001000000000000ULL	
//////////////////////////////bits below here are used in values[] of tagger  0x0000 are used in tagger internals...
#define PARTICLE				0x0000800000000000ULL	//   multiword separable verb (full verb marked w systemflag PHRASAL_VERB
#define STARTTAGS				0x0000800000000000ULL

#define TO_INFINITIVE	 		0x0000400000000000ULL	// 20th horse
#define VERB_PRESENT			0x0000200000000000ULL	// present plural (usually infinitive)
#define VERB_PRESENT_3PS		0x0000100000000000ULL	// 3rd person singular singular
#define VERB_PRESENT_PARTICIPLE 0x0000080000000000ULL	// GERUND,  Pennbank: VBG
#define VERB_PAST				0x0000040000000000ULL	// Pennbank: VBD
#define VERB_PAST_PARTICIPLE    0x0000020000000000ULL	// Pennbank VBN
#define VERB_INFINITIVE			0x0000010000000000ULL	//   all tense forms are linked into a circular ring
#define NOUN_INFINITIVE			0x0000008000000000ULL	
#define AUX_VERB_PASTPARTICIPLE 0x0000004000000000ULL	// "been" as aux_verb_pastparticiple
#define	PREDETERMINER			0x0000002000000000ULL	

#define AUX_VERB				0x0000001000000000ULL	//   auxilliary verb Pennbank MD
#define AUX_VERB_PRESENT		0x0000000800000000ULL
#define AUX_VERB_FUTURE			0x0000000400000000ULL
#define AUX_VERB_PAST			0x0000000200000000ULL
#define AUX_VERB_TENSES ( AUX_VERB_PRESENT | AUX_VERB_FUTURE | AUX_VERB_PAST | AUX_VERB_PASTPARTICIPLE )

#define NOUN_ORDINAL 			0x0000000100000000ULL

#define CONJUNCTION_BITS	 ( CONJUNCTION_COORDINATE | CONJUNCTION_SUBORDINATE )
#define DETERMINER_BITS		( DETERMINER | PREDETERMINER )
#define PRONOUN_BITS	( PRONOUN_OBJECT | PRONOUN_SUBJECT | PRONOUN_POSSESSIVE )
#define PART_OF_SPEECH			( AUX_VERB | ESSENTIAL_FLAGS | DETERMINER | PREDETERMINER | PRONOUN_BITS | CONJUNCTION_COORDINATE | CONJUNCTION_SUBORDINATE  ) 
// + particles (which we treat as derived from prepositions) + interjections 
#define POSSESSIVE_BITS ( PRONOUN_POSSESSIVE | POSSESSIVE )
#define ADVERB_BITS	 ( ADVERB_BASIC | ADVERB_MORE | ADVERB_MOST | WH_ADVERB )
#define NOUN_TITLE		 (  NOUN | NOUN_TITLE_OF_WORK | NOUN_PROPER_SINGULAR )
#define NOUN_HUMANGROUP		( NOUN | NOUN_HUMAN | NOUN_TITLE_OF_WORK | NOUN_THEY )
#define NOUN_MALEHUMAN     ( NOUN | NOUN_HUMAN | NOUN_HE | NOUN_PROPER_SINGULAR )
#define NOUN_FEMALEHUMAN    ( NOUN |  NOUN_HUMAN | NOUN_SHE | NOUN_PROPER_SINGULAR )
#define NOUN_HUMANNAME ( NOUN_HUMAN | NOUN | NOUN_PROPER_SINGULAR )
#define NOUN_PROPERTIES ( NOUN_ROLE | NOUN_MASS | NOUN_HE | NOUN_THEY | NOUN_SINGULAR | NOUN_PLURAL | NOUN_PROPER_SINGULAR | NOUN_PROPER_PLURAL | NOUN_ABSTRACT | NOUN_HUMAN | NOUN_FIRSTNAME | NOUN_SHE | NOUN_TITLE_OF_ADDRESS | NOUN_TITLE_OF_WORK  )
#define NUMBER_BITS ( NOUN_CARDINAL | NOUN_ORDINAL | ADJECTIVE_CARDINAL | ADJECTIVE_ORDINAL )
#define VERB_TENSES (  VERB_INFINITIVE | VERB_PRESENT | VERB_PRESENT_3PS | VERB_PAST | VERB_PAST_PARTICIPLE | VERB_PRESENT_PARTICIPLE  )
#define VERB_PROPERTIES (  VERB_TENSES )
#define ADJECTIVE_BITS ( ADJECTIVE_BASIC | ADJECTIVE_MORE | ADJECTIVE_MOST | ADJECTIVE_PARTICIPLE | ADJECTIVE_CARDINAL | ADJECTIVE_ORDINAL )
#define NOUN_BITS ( NOUN_SINGULAR | NOUN_PLURAL | NOUN_PROPER_SINGULAR  | NOUN_PROPER_PLURAL | NOUN_GERUND | NOUN_CARDINAL | NOUN_ORDINAL )
#define TAG_TEST ( COMMA | PAREN | PARTICLE | VERB_TENSES | NOUN_BITS | FOREIGN_WORD | NOUN_INFINITIVE | PREDETERMINER | DETERMINER | ADJECTIVE_BITS | AUX_VERB_TENSES | ADVERB_BITS  | PRONOUN_BITS | CONJUNCTION_BITS | POSSESSIVE | THERE_EXISTENTIAL | PREPOSITION | TO_INFINITIVE )
#define NOUN_DESCRIPTION_BITS ( ADJECTIVE_BITS | DETERMINER  | PREDETERMINER  | ADVERB_BITS  | PRONOUN_POSSESSIVE  | POSSESSIVE | NOUN_BITS )
#define FULL_NOUN_BITS ( NOUN_BITS | NOUN_INFINITIVE )
#define NOUN_PROPER ( NOUN_PROPER_SINGULAR | NOUN_PROPER_PLURAL | NOUN_PROPERTIES )

// SYSTEMFLAGS
#define PATTERN_WORD		0x0000000000000001ULL	//   this word is known to the topic system as a keyword (may or may not be a real word)
#define FUNCTION_NAME		0x0000000000000002ULL	//   name of a ^function  (has non-zero ->x.codeIndex if system, else is user) only applicable to ^ words
#define DELETED_WORD		0x0000000000000004ULL	//   want to not use this word
#define CONCEPT				0x0000000000000008ULL	// topic or concept has been read via a definition
#define IS_PATTERN_MACRO	0x0000000000000010ULL	//   pattern vs output use
#define BUILD0				0x0000000000000020ULL// comes from build0 data (marker on functions, concepts, topics)
#define BUILD1				0x0000000000000040ULL	// comes from build1 data
#define QUERY_KIND			0x0000000000000080ULL	// is a query item
//			0x0000000000000100ULL  //   is the verb is -- MATCH IN FACT by being unused
//			0x0000000000000200ULL  //   is the verb member  -- MATCH IN FACT by being unused
#define PATTERN_ALLOWED		0x0000000000000400ULL	// dont warn about spelling during script compile
#define SUBSTITUTE			0x0000000000000800ULL  //   word has substitute attached 
#define TOPIC_NAME			0x0000000000001000ULL	//  this is a ~xxxx topic name in the system - only applicable to ~ words
#define GRADE5_6            0X0000000000002000ULL //   age 10-11
#define GRADE3_4            0X0000000000004000ULL //   age 8-9
#define GRADE1_2            0x0000000000008000ULL //   age 6-7
#define KINDERGARTEN        0x0000000000010000ULL //   age 5
#define TIMEWORD			0x0000000000020000ULL
#define SPACEWORD			0x0000000000040000ULL
#define PHRASAL_VERB		0x0000000000080000ULL
#define UPPERCASE_HASH		0x0000000000100000ULL //   uses an upper case bucket  --- on MAIN WORDS
#define LINKING_VERB		0x0000000000200000ULL	// allows adjective complement
#define VAR_CHANGED			0x0000000000400000ULL  // applicable only to $variables
#define CONJUNCT_SUBORD_NOUN 0x0000000000400000ULL	// (ordinary words) this might be a noun clause starter
#define IN_SENTENCE			0x0000000000800000ULL
#define OTHER_SINGULAR		0x0000000001000000ULL // for pronouns, etc
#define OTHER_PLURAL		0x0000000002000000ULL
#define SUBSTITUTE_RECIPIENT 0x0000000004000000ULL
#define SAFETOPIC			 0x0000000008000000ULL

// CANNOT REFERENCE BELOW IN POS RULES
#define VERB_CONJUGATE1				0x8000000000000000ULL	//   1st word of composite verb conjugates  (e.g. peter_out)
#define VERB_CONJUGATE2				0x4000000000000000ULL	//   2nd word of composite verb conjugates  (e.g., re-energize)
#define VERB_CONJUGATE3				0x2000000000000000ULL	//	3rd word of composite verb conjugates	

#define PREP_LOCATION				0x1000000000000000ULL
#define PREP_TIME					0x0800000000000000ULL
#define PREP_RELATION				0x0400000000000000ULL
#define PREP_INTRODUCTION			0x0200000000000000ULL
#define INTERNAL_MARK				0x0100000000000000ULL
#define HAS_EXCLUDE					0x0080000000000000ULL
#define	APPUSAGE_1					0x0040000000000000ULL
#define IS_TABLE_MACRO				0x0020000000000000ULL
//									0x0010000000000000ULL
//									0x0008000000000000ULL
//									0x0004000000000000ULL
//									0x0002000000000000ULL
#define MARKED_WORD					0x0001000000000000ULL  // same bit value as punctuation-- NO ONE will assign punctuation via script AddProperty
// CANNOT REFERNCE ABOVE IN POS TAGGING RULES
#define NO_EXTENDED_WRITE_FLAGS ( PATTERN_WORD | FUNCTION_NAME | DELETED_WORD | IS_TABLE_MACRO | IS_PATTERN_MACRO |  BUILD0 | BUILD1 | QUERY_KIND | PATTERN_ALLOWED | TOPIC_NAME | SUBSTITUTE )

#define ALLOW_INFINITIVE			0x0000800000000000ULL	// supports infinitive must not use top byte else hasproperty messes up
#define EXISTENTIAL_BE				0x0000400000000000ULL	// "there is or there seem"
#define EXTENT_ADVERB				0x0000200000000000ULL	
#define ADJECTIVE_POSTPOSITIVE		0x0000100000000000ULL	// adjective can occur AFTER the noun
#define PREDETERMINER_TARGET		0x0000080000000000ULL	// predeterminer can come before these (a an the)
#define OMITTABLE_TIME_PREPOSITION	0x0000040000000000ULL // can be used w/o preposition
#define VERBS_ACCEPTING_OF_AFTER	0x0000020000000000ULL // of can follow this verb
#define SYSTEM_MORE_MOST			0x0000010000000000ULL	// more most, less least

#define VERB_DIRECTOBJECT			0x0000008000000000ULL	// verb can take an object (no verb accepting an object REFUSED no object)
#define VERB_INDIRECTOBJECT			0x0000004000000000ULL	// verb can take indirect object  
#define POTENTIAL_CLAUSE_STARTER	0x0000002000000000ULL	// relative pronoun or adjective clause marker
//		0x0000001000000000ULL
#define NOUN_BEING					0x0000000800000000ULL

#define NOUN_PHRASE_BITS		( DETERMINER | ADJECTIVE_BITS | ADVERB_BITS | NOUN_SINGULAR | NOUN_PLURAL | NOUN_PROPER_SINGULAR | NOUN_PROPER_PLURAL | NOUN_CARDINAL )

// adult is all the rest
#define AGE_LEARNED ( KINDERGARTEN | GRADE1_2 | GRADE3_4 | GRADE5_6 )


// flags on facts

// USER FLAGS: 0xffff0000ULL

#define USER_FLAGS			0xFFFF0000

// transient flags
#define MARKED_FACT         0x00008000  //   TRANSIENT : used during inferencing sometimes to see if fact is marked
#define MARKED_FACT2        0x00004000  //   TRANSIENT: used during inferencing sometimes to see if 2ndary fact is marked
#define FACTDEAD			0x00002000   //   has been killed off
#define FACTTRANSIENT       0x00001000  //   save only with a set, not with a user or base syste,

//#define UNUSEDF5	        0x00000800  //  dont match on canonicals
//#define UNUSEDF4	        0x00000400  //  dont match on canonicals
//#define FACT_MEMBER		0x00000200  //   is the verb member  -- MATCH IN FACT by being unused
//#define FACT_IS			0x00000100  //   is the verb is -- MATCH IN FACT by being unused

#define FACTSUBJECT         0x00000080  //   index is - relative number to fact 
#define FACTVERB			0x00000040	//   is 1st in its bucket (transient flag for read/WriteBinary) which MIRRORS DICT BUCKETHEADER flag: 
#define FACTOBJECT		    0x00000020  //   does not apply to canonical forms of words, only the original form - for classes and sets, means dont chase the set
#define FACTDUPLICATE		0x00000010	//   allow repeats of this face

//#define UNUSEDF3	        0x00000008  //  dont match on canonicals
//#define UNUSEDF2	        0x00000004  //  dont match on canonicals
#define ORIGINAL_ONLY       0x00000002  //  dont match on canonicals
//#define UNUSEDF1	        0x00000001  //  dont match on canonicals


// topic control flags

// permanent flags 
#define TOPIC_NOERASE 1     //   do not erase as you go
#define TOPIC_REPEAT 2      //   allow repeat output
#define TOPIC_RANDOM 4      //   random access responders (not gambits)
#define TOPIC_SYSTEM 8		//   
#define TOPIC_NOSTAY 16    //   do not stay in this topic
#define TOPIC_PRIORITY 32	//   prefer this 
#define TOPIC_LOWPRIORITY 64 //   deprefer this
#define TOPIC_NOBLOCKING 128 //   :verify should not complain about blocking

//   TRANSIENT FLAGS
#define TOPIC_BLOCKED 256		//   (transient per user) disabled by some users 
#define TOPIC_USED 512			//   (transient per user) indicates need to write out the topic
#define TOPIC_VOLLEY 1024		//   (transient per user) gambit issues this round from this topic

// pos tagger result operators
#define DISCARD 1
#define KEEP 2
#define TRACE 8

// pos tagger pattern values   5 bits (0-31) + 3 flags
#define HAS 1 // any bit matching 
#define IS 2	// is exactly this
#define INCLUDE 3 // has bit AND has other bits
#define ISWORD 4 // is this word
#define POSITION 5 // sentence boundary
#define START POSITION
#define END POSITION
#define ISCANONICAL 6 // canonical word is this
#define ISMEMBER 7	// is member of this set
#define PARTICLEVERB 8	// verb before this forms a phrasal verb
#define ISADJACENT 9	// before or after is this
#define ISCLAUSE 10	//  has noun and verb freestanding
#define HASPROPERTY 11
#define ISROLE	12	
#define ENDSWITH 13			// suffix it ends in
#define LACKSROLE 14		// works when tagging has been tried
#define HASCANONICALPROPERTY 15
#define LACKSPRECEEDINGAUXVERB	16
#define ISQUESTION 17
#define ISABSTRACT 18
#define NOFOLLOWING 19
#define ORIGINAL 20
#define POSSIBLEMASSNOUN 21
#define NOFOLLOWINGNOUN 22
#define PRIORCANONICAL 23
#define ISQWORD 24
#define ISJOINER 25
#define ISLOWERCASE 26
#define LASTCONTROL ISLOWERCASE  // add new ops to optable as well
	
#define SKIP 1 // if it matches, move ptr along, if it doesnt DONT
#define STAY 2
#define NOTCONTROL 4 


// pos tagger roles and states
#define MAINSUBJECT		1			// 0x000001			// noun roles like Mainsubject etc are ordered lowest first for pronoun IT priority
#define MAINVERB	2				// 0x000002
#define MAINOBJECT		4			// 0x000004
#define MAININDIRECTOBJECT		8	// 0x000008
#define SUBJECT2	16				// 0x000010
#define VERB2		32				// 0x000020
#define OBJECT2		64				// 0x000040
#define INDIRECTOBJECT2		128		// 0x000080
#define CLAUSE 256					// 0x000100
#define PREPPHRASE 512				// 0x000200
#define VERBAL 1024					// 0x000400
#define CONJUNCT_WORD 2048			// 0x000800
#define CONJUNCT_PHRASE 4096		// 0x001000
#define CONJUNCT_CLAUSE 8192		// 0x002000
#define CONJUNCT_SENTENCE 16384		// 0x004000
#define COMMA_PHRASE	32768		// 0x008000
#define APPOSITIVE		65536		// 0x010000 (can lead or follow)
#define OBJECT_COMPLEMENT		       0x020000	// can be noun, or adjective after object...
#define ADJECTIVE_AS_OBJECT			   0x040000
#define NOT							   0x080000
#define ADJECTIVE_NOUN				   0x100000
#define OMITTED_TIME_PREP			   0x200000
#define ADDRESS_START				   0x400000
#define ABSOLUTE_PHRASE				   0x800000

#define KINDS_OF_PHRASES ( CLAUSE | PREPPHRASE | VERBAL | ABSOLUTE_PHRASE )

// pos tagger ZONE roles for a comma zone
#define ZONE_SUBJECT			0x000001	// noun before any verb
#define ZONE_VERB				0x000002
#define ZONE_OBJECT				0x000004	// noun AFTER a verb
#define ZONE_CONJUNCT			0x000008	// coord or subord conjunction
#define ZONE_FULLVERB			0x000010	// has normal verb tense or has aux
#define ZONE_PCV				0x000020	// zone is entirely phrases, clauses, and verbals
#define ZONE_ADDRESS			0x000040	// zone is an addressing name start. "Jim, you suck."
#define ZONE_AUX				0x000080	// there is aux in the zone
#define ZONE_ABSOLUTE			0x000100	// absolute zone has subject and partial participle verb, used to describe noun in another zone
#define ZONE_AMBIGUOUS			0x000200	// zone is not resolved yet

// control over tokenization
#define DO_SUBSTITUTES 1		//0x000001
#define DO_NUMBER_MERGE 2		//0x000002
#define DO_PROPERNAME_MERGE 4	//0x000004
#define DO_SPELLCHECK 8			//0x000008
			//0x000010  NOW UNUSED DO_UTF8_CONVERT
#define DO_INTERJECTION_SPLITTING 32 //0x000020
#define DO_POSTAG 64			//0x000040
#define DO_PARSE 128			//0x000080
#define NO_IMPERATIVE 256		//0x000100
#define NO_VERB 512				//0x000200
#define NO_WITHIN 1024			//0x000400
#define NO_SENTENCE_END 2048	//0x000800
#define DO_CONDITIONAL_POSTAG 4096 //0x001000
#define AUTOWHY	8192			//0x002000 // a rejoinder control, not a token control
#define DO_NOPARSEGESTURE 16384 //0x004000

#define    OUTPUT_ONCE 1 
#define    OUTPUT_KEEPSET  2  //   dont expand class or set
#define    OUTPUT_KEEPVAR  4 //   dont expand a var
#define    OUTPUT_KEEPQUERYSET 8 //   dont expand a fact var like @1object
#define    OUTPUT_SILENT 16		// dont show stuff if trace is on
#define    OUTPUT_EVALCODE  32	// process code strings
#define    OUTPUT_NOCOMMANUMBER 64 // dont add to numbers
#define    OUTPUT_NOTREALBUFFER  128	// dont back up past start at all
#define    OUTPUT_NOQUOTES  256	 // strip quotes off quoted strings
#define	   OUTPUT_LOOP 512		// coming from a loop
#define	   OUTPUT_UNTOUCHEDSTRING 1024 // leave string as argument

//   values of TokenMarkers (seen processing input) 
#define QUESTIONMARK 1				// 0x00001    
#define EXCLAMATIONMARK 2			// 0x00002   
#define NOTFLAG 4					// 0x00004    
#define USERINPUT 8					// 0x00008    
#define PASSIVE 16					// 0x00010    
#define AMBIGUOUS_POS 32			// 0x00020   
#define FAULTY_PARSE 64				// 0x00040    
#define PRESENT 128					// 0x00080    
#define PAST 256					// 0x00100    
#define FUTURE 512					// 0x00200    
#define SIMPLE_TENSE		1024	// 0x00400    
#define CONTINUOUS		2048		// 0x00800    
#define PERFECT			4096		// 0x01000   
#define PERFECT_CONTINUOUS 8192		// 0x02000    
#define FOREIGN_INPUT	16384		// 0x04000

struct WORDENTRY;
typedef WORDENTRY* WORDP;

typedef void (*DICTIONARY_FUNCTION)(WORDP D, void* data);

struct FACT;
typedef unsigned int MEANING; //   a flagged indexed dict ptr

typedef unsigned int FACTOID; //   a fact index

typedef struct WORDENTRY //   a dictionary entry  - starred items are written to the dictionary
{
	uint64  properties;				//   description of this node 
	uint64	hash;					//   we presume 2 hashs never collide, so we dont check the string for matching
	uint64  systemFlags;			//   non dictionary properties of the entry	- can vary based on :build state
	uint64  tried;					//   bits of which meaning the inferMark applies to

	//   if you edit this, you may need to change ReadBinaryEntry and WriteBinaryEntry
	char*     word;				//   entry name
  	MEANING*  meanings;			//   list of meanings (synsets) of this word - Will be wordnet synset id OR self ptr -- 1-based since 0th meaning means all meanings
#ifndef NOGLOSS
	MEANING*  glosses;			//   list of glosses for synset head meanings - is offset to allocstring and id index of meaning involved.
#endif

	//   related members of this word (circular lists) 
	WORDP			conjugation;	//   conjugation list for irregular verbs 
	WORDP			plurality;		//   plurality list for irregular nouns	
	WORDP			comparison;			//   comparison list (start with alpha) for irregular adjective/adverb

	union {
		char* topicRestriction;		//   for topic name (start with $) - bot topic applies to
	    char* fndefinition;			//   for macro name (start with ^) - if systemflag is on, is user script
	    char* userValue;			//   if a $uservar (start with $) OR if a search label uservar 
		WORDP substitutes;			//   words (with systemflag SUBSTITUTE) that should be adjusted to during tokenization
	}w;
		
	FACTOID subjectHead;		//  start threads for facts run thru here 
	FACTOID verbHead;			//  start threads for facts run thru here 
	FACTOID objectHead;			//  start threads for facts run thru here 
	FACTOID	factBack;			//	backtrace thread on search

    union {
          unsigned short topicIndex;		//   for a ~topic or %systemVariable, this is its id
		  unsigned short codeIndex;		//   for a function, its the table index for it
    }x;
	
	union {
          unsigned short length;		//  length of the word
    }y;

  	unsigned int nextNode;		//   bucket-link for dictionary hash + top bye GETMULTIWORDHEADER // can this word lead a phrase to be joined - can vary based on :build state -- really only needs 4 bits
    unsigned int  inferMark;     //   been here marker during marking facts, inferencing (for propogation) and during scriptcompile 

    //   these two fields used in pattern matching set up once
    union {
        unsigned int  patternStamp;  //   is this found in sentence (current timestamp, in which case whereInSentence is locations)
        unsigned int  argumentCount; //   for function defintions - if systemflag is on
    }v;


} WORDENTRY;



#include "dictionaryMore.h"

#endif
